"""A cleanup task for removing past runs."""

from orchestrator.db import db
from orchestrator.targets import Target
from orchestrator.workflow import StepList, begin, done, step, workflow

from gso.services.processes import get_all_cleanup_tasks, get_created_and_completed_processes_by_id


@step("Remove old cleanup tasks")
def remove_old_cleanup_tasks() -> None:
    """Find and remove runs of old cleanup tasks."""
    cleanup_tasks = get_all_cleanup_tasks()

    for cleanup_task in cleanup_tasks:
        tasks = get_created_and_completed_processes_by_id(cleanup_task.workflow_id)

        for task in tasks:
            db.session.delete(task)


@workflow("Remove old cleanup tasks", target=Target.SYSTEM)
def task_clean_old_tasks() -> StepList:
    """Remove all runs of old cleanup tasks.

    This will look for all past executions of the ``orchestrator-core`` built-in cleanup task, and this current cleanup
    task. Once they have run to completion, or are stuck in a "created" state, they serve no purpose in the database and
    can therefore be removed.
    """
    return begin >> remove_old_cleanup_tasks >> done
